<?php

declare(strict_types=1);

namespace Drupal\fs_integration;

use Drupal\fs_integration\Event\StudyPlanQueryEvent;
use Frontkom\GraphqlClient\Query;
use GraphQL\InlineFragment;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * A service for the FS GraphQL API.
 */
final class FsGraphqlService implements FsGraphqlServiceInterface {

  /**
   * Constructs a FsGraphqlClient object.
   */
  public function __construct(
    private readonly FsGraphqlClient $client,
    private readonly EventDispatcherInterface $eventDispatcher,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function getStudyPlanById(string $id): StudyPlan {
    $query = $this->getStudyPlanQueryForId($id);
    $event = new StudyPlanQueryEvent($query);
    $this->eventDispatcher->dispatch($event);
    $response = $this->client->runQuery($query);
    return StudyPlan::createFromFsData($response->getData());
  }

  /**
   * {@inheritdoc}
   */
  public function getStudyPlanQueryForId(string $id) : Query {
    return (new Query('node'))
      ->setArguments(['id' => $id])
      ->setSelectionSet([
        'id',
        (new InlineFragment('StudieoppbygningForKull'))
          ->setSelectionSet([
            'id',
            (new Query('studieprogram'))
              ->setSelectionSet([
                'kode',
              ]),
            (new Query('gjelderKullFra'))
              ->setSelectionSet([
                'arstall',
                (new Query('betegnelse'))
                  ->setSelectionSet([
                    'kode',
                  ]),
              ]),
            (new Query('alleOppbygningsdeler'))
              ->setSelectionSet([
                'id',
                (new Query('parent'))
                  ->setSelectionSet([
                    'id',
                  ]),
                (new Query('studieoppbygningsdel'))
                  ->setSelectionSet([
                    'kode',
                    'id',
                    (new Query('navnAlleSprak'))
                      ->setSelectionSet([
                        'nb',
                        'en',
                      ]),
                    (new Query('emner'))
                      ->setSelectionSet([
                        'rekkefolgenummer',
                        (new Query('terminForPlasseringIUtdanningsplan'))
                          ->setSelectionSet([
                            'terminnummerForhandsvalgt',
                            'terminnummerFra',
                            'terminnummerTil',
                          ]),
                        (new Query('emne'))
                          ->setSelectionSet([
                            'kode',
                            (new Query('navnAlleSprak'))
                              ->setSelectionSet([
                                'en',
                                'nb',
                              ]),
                            (new Query('vekting'))
                              ->setSelectionSet([
                                (new Query('emnevekting'))
                                  ->setSelectionSet([
                                    'verdi',
                                    (new Query('vektingstype'))
                                      ->setSelectionSet([
                                        'kode',
                                      ]),
                                  ]),
                              ]),
                          ]),
                        (new Query('emnekategori'))
                          ->setSelectionSet([
                            'kode',
                          ]),
                      ]),
                  ]),
              ]),
          ]),
      ]);
  }

}
